package com.malk.lanyun.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.malk.lanyun.entity.SqlServerBean;
import com.malk.lanyun.service.NCService;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import com.malk.utils.UtilDateTime;
import com.malk.utils.UtilMap;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class NCServiceImpl implements NCService {

    @Autowired
    private YDClient ydClient;

    @Autowired
    private SqlServerBean sqlBean;

    @Autowired
    private YDService ydService;

    // 获取当前日期
    LocalDate lastMonth = LocalDate.now().minusMonths(1);
    //本月
//    LocalDate lastMonth = LocalDate.now();
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
    @Override
    public void getNcInfo() {
        // 获取上月年，月
        int year = lastMonth.getYear();
        String month = String.format("%02d",lastMonth.getMonthValue()) ;
        deleteMonthDate("FORM-D7FAD41DA72041ED96431F4219F082B7DTNH","textField_m0dkzhwo", String.valueOf(year),month);

        Connection connection = null;
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            // 加载 JDBC 驱动
            Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
            // 建立连接
            connection = DriverManager.getConnection(sqlBean.getUrl(), sqlBean.getUsername(), sqlBean.getPassword());
            // 创建 Statement 对象
            statement = connection.createStatement();

            // 执行查询
            String sql = "SELECT b.adjustperiod AS adjustperiod, b.pk_accasoa AS pk_accasoa, a.name AS accasoa_name, SUM ( b.creditamount ) AS creditamount\n" +
                    "            FROM gl_detail b JOIN bd_accasoa a ON b.pk_accasoa = a.pk_accasoa WHERE  b.yearv = "+year+"\n AND b.adjustperiod = "+ month +
                    "            AND b.accountcode IN ( '50010101', '500102', '500103', '500117', '500114', '500115', '500118', '500119', '500109', '500133' )\n" +
                    "            AND b.pk_accountingbook IN ( SELECT pk_accountingbook FROM org_accountingbook WHERE code IN ( 'A01-0004', 'A02-0004', 'A03-0004', 'A04-0004', 'A05-0004', 'A06-0004', 'A10-0004', 'A11-0004' ) )\n" +
                    "            GROUP BY b.adjustperiod,  b.pk_accasoa, a.name";

            resultSet = statement.executeQuery(sql);

            // 处理结果集
            while (resultSet.next()) {
                Map hashMap = new HashMap();

                String yearAndNonth = year +"-"+resultSet.getString("adjustperiod");
                hashMap.put("dateField_m0dkzhwp", UtilDateTime.parse(yearAndNonth,"yyyy-MM"));
                hashMap.put("textField_m0dkzhwo", yearAndNonth);
                hashMap.put("textField_m0dkzhwu",resultSet.getString("pk_accasoa"));
                hashMap.put("textField_m0dkzhwv",resultSet.getString("accasoa_name"));
                hashMap.put("numberField_m0dkzhx1",resultSet.getBigDecimal("creditamount"));

                log.info("yearv："+UtilDateTime.parse(yearAndNonth,"yyyy-MM"));
                log.info("periodv："+resultSet.getString("pk_accasoa"));
                log.info("periodv："+resultSet.getString("accasoa_name"));
                log.info("creditamount："+resultSet.getBigDecimal("creditamount"));

                //写入月度项目经营情况
                ydClient.operateData(YDParam.builder()
                        .appType("APP_ERBDTFS82HOVBPL3NFH0")
                        .systemToken("RRB66F91T97H1WN89QZYC47PKLZO2ZQOUMOQLP")
                        .formUuid("FORM-D7FAD41DA72041ED96431F4219F082B7DTNH")
                        .formDataJson(JSON.toJSONString(hashMap))
                        .build(), YDConf.FORM_OPERATION.create);
            }
        } catch (Exception e) {
            log.info("数据库连接异常"+e.toString());
            e.printStackTrace();
        } finally {
            // 关闭资源
            try {
                if (resultSet != null) resultSet.close();
                if (statement != null) statement.close();
                if (connection != null) connection.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void getNcCgInfo() {
        // 获取当前年
        int year = lastMonth.getYear();
        String month = String.format("%02d",lastMonth.getMonthValue()) ;
        deleteMonthDate("FORM-EC67BCB875264EE9AE25C9D663568CAFCYU6","textField_m0dkzhwo", String.valueOf(year),month);

        Connection connection = null;
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            // 加载 JDBC 驱动
            Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
            // 建立连接
            connection = DriverManager.getConnection(sqlBean.getUrl(), sqlBean.getUsername(), sqlBean.getPassword());

            // 创建 Statement 对象
            statement = connection.createStatement();

            // 测试sql 执行查询
            String sql = "SELECT b.adjustperiod AS adjustperiod, b.pk_accasoa AS pk_accasoa,a.name AS accasoa_name, SUM ( b.creditamount ) AS creditamount\n" +
                    "            FROM gl_detail b JOIN bd_accasoa a ON b.pk_accasoa = a.pk_accasoa WHERE b.yearv = "+year+" AND b.adjustperiod = "+ month +
                    "            AND b.accountcode IN ( '500114', '500115' )\n" +
                    "            AND b.pk_accountingbook IN ( SELECT pk_accountingbook FROM org_accountingbook WHERE code IN ( 'A01-0004', 'A02-0004', 'A03-0004', 'A04-0004', 'A05-0004', 'A06-0004', 'A10-0004', 'A11-0004' ))\n" +
                    "            GROUP BY b.adjustperiod,  b.pk_accasoa,  a.name";


            resultSet = statement.executeQuery(sql);

            while (resultSet.next()) {
                Map hashMap = new HashMap();

                String yearAndNonth = year +"-"+resultSet.getString("adjustperiod");
                hashMap.put("dateField_m0dkzhwp", UtilDateTime.parse(yearAndNonth,"yyyy-MM"));
                hashMap.put("textField_m0dkzhwo", yearAndNonth);
                hashMap.put("textField_m0dkzhwu",resultSet.getString("pk_accasoa"));
                hashMap.put("textField_m0dkzhwv",resultSet.getString("accasoa_name"));
                hashMap.put("numberField_m0dkzhx1",resultSet.getBigDecimal("creditamount"));

                log.info("yearv："+UtilDateTime.parse(yearAndNonth,"yyyy-MM"));
                log.info("periodv："+resultSet.getString("pk_accasoa"));
                log.info("periodv："+resultSet.getString("accasoa_name"));
                log.info("creditamount："+resultSet.getBigDecimal("creditamount"));

                ydClient.operateData(YDParam.builder()
                        .appType("APP_ERBDTFS82HOVBPL3NFH0")
                        .systemToken("RRB66F91T97H1WN89QZYC47PKLZO2ZQOUMOQLP")
                        .formUuid("FORM-EC67BCB875264EE9AE25C9D663568CAFCYU6")
                        .formDataJson(JSON.toJSONString(hashMap))
                        .build(), YDConf.FORM_OPERATION.create);
            }
        } catch (Exception e) {
            log.info("数据库连接异常"+e.toString());
            e.printStackTrace();

        } finally {
            // 关闭资源
            try {
                if (resultSet != null) resultSet.close();
                if (statement != null) statement.close();
                if (connection != null) connection.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 大业主项目
     */
    @Override
    public void getNcCost() {
        // 获取当前年
        int year = lastMonth.getYear();
        String month = String.format("%02d",lastMonth.getMonthValue()) ;
        deleteMonthDate("FORM-87BF51F1E54D4D73A987939613D07BD5ZZE9","textField_m0dkzhwo", String.valueOf(year),month);

        Connection connection = null;
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            // 加载 JDBC 驱动
            Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
            // 建立连接
            connection = DriverManager.getConnection(sqlBean.getUrl(), sqlBean.getUsername(), sqlBean.getPassword());

            // 创建 Statement 对象
            statement = connection.createStatement();

            // 测试sql 执行查询
            String sql = "SELECT b.adjustperiod AS adjustperiod, b.pk_accasoa AS pk_accasoa,a.name AS accasoa_name, SUM ( b.creditamount ) AS creditamount\n" +
                    "            FROM gl_detail b JOIN bd_accasoa a ON b.pk_accasoa = a.pk_accasoa WHERE b.yearv = "+year+" AND b.adjustperiod = "+ month +
                    "            AND b.accountcode IN ( '500114', '500115' )\n" +
                    "            AND b.pk_accountingbook IN ( SELECT pk_accountingbook FROM org_accountingbook WHERE code IN ( 'A01-0004', 'A02-0004', 'A03-0004', 'A04-0004', 'A05-0004', 'A06-0004', 'A10-0004', 'A11-0004' ))\n" +
                    "            GROUP BY b.adjustperiod,  b.pk_accasoa,  a.name";


            resultSet = statement.executeQuery(sql);

            while (resultSet.next()) {
                Map hashMap = new HashMap();

                String yearAndNonth = year +"-"+resultSet.getString("adjustperiod");
                hashMap.put("dateField_m0dkzhwp", UtilDateTime.parse(yearAndNonth,"yyyy-MM"));
                hashMap.put("textField_m0dkzhwo", yearAndNonth);
                hashMap.put("textField_m0dkzhwu",resultSet.getString("pk_accasoa"));
                hashMap.put("textField_m0dkzhwv",resultSet.getString("accasoa_name"));
                hashMap.put("textField_m0eudqp0","大业主项目ID");
                hashMap.put("textField_m0eudqp2","大业主项目名称");
                hashMap.put("numberField_m0dkzhx1",resultSet.getBigDecimal("creditamount"));

                log.info("yearv："+UtilDateTime.parse(yearAndNonth,"yyyy-MM"));
                log.info("periodv："+resultSet.getString("pk_accasoa"));
                log.info("periodv："+resultSet.getString("accasoa_name"));
                log.info("creditamount："+resultSet.getBigDecimal("creditamount"));

                ydClient.operateData(YDParam.builder()
                        .appType("APP_ERBDTFS82HOVBPL3NFH0")
                        .systemToken("RRB66F91T97H1WN89QZYC47PKLZO2ZQOUMOQLP")
                        .formUuid("FORM-87BF51F1E54D4D73A987939613D07BD5ZZE9")
                        .formDataJson(JSON.toJSONString(hashMap))
                        .build(), YDConf.FORM_OPERATION.create);
            }
        } catch (Exception e) {
            log.info("数据库连接异常"+e.toString());
            e.printStackTrace();
        } finally {
            // 关闭资源
            try {
                if (resultSet != null) resultSet.close();
                if (statement != null) statement.close();
                if (connection != null) connection.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void getNcCollection() {
        // 获取当前年
        int year = lastMonth.getYear();
        String month = String.format("%02d",lastMonth.getMonthValue()) ;
        deleteMonthDate("FORM-5654B7C22E7546E9BBE7B4EF46DE08BAXZXU","textField_m0dkzhwo", String.valueOf(year),month);

        System.out.println(month);
        Connection connection = null;
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            // 加载 JDBC 驱动
            Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
            // 建立连接
            connection = DriverManager.getConnection(sqlBean.getUrl(), sqlBean.getUsername(), sqlBean.getPassword());

            // 创建 Statement 对象
            statement = connection.createStatement();

            // 测试sql 执行查询
            String sql = "SELECT b.adjustperiod AS adjustperiod, b.pk_accasoa AS pk_accasoa,a.name AS accasoa_name, SUM ( b.creditamount ) AS creditamount\n" +
                    "            FROM gl_detail b JOIN bd_accasoa a ON b.pk_accasoa = a.pk_accasoa WHERE b.yearv = "+year+" AND b.adjustperiod = '"+ month + "'" +
                    "            AND (b.accountcode like '6602%' OR b.accountcode IN ('660101','660103','660104'))" +
                    "            AND b.pk_accountingbook IN ( SELECT pk_accountingbook FROM org_accountingbook WHERE code IN ( 'A01-0004', 'A02-0004', 'A03-0004', 'A04-0004', 'A05-0004', 'A06-0004', 'A10-0004', 'A11-0004' ))\n" +
                    "            GROUP BY b.adjustperiod,  b.pk_accasoa,  a.name";


            resultSet = statement.executeQuery(sql);

            while (resultSet.next()) {
                Map hashMap = new HashMap();

                String yearAndNonth = year +"-"+resultSet.getString("adjustperiod");
                hashMap.put("dateField_m0dkzhwp", UtilDateTime.parse(yearAndNonth,"yyyy-MM"));
                hashMap.put("textField_m0dkzhwo", yearAndNonth);
                hashMap.put("textField_m0dkzhwu",resultSet.getString("pk_accasoa"));
                hashMap.put("textField_m0dkzhwv",resultSet.getString("accasoa_name"));
                hashMap.put("textField_m0eudqp0","部门");
                hashMap.put("numberField_m0dkzhx1",resultSet.getBigDecimal("creditamount"));

                log.info("yearv："+UtilDateTime.parse(yearAndNonth,"yyyy-MM"));
                log.info("periodv："+resultSet.getString("pk_accasoa"));
                log.info("periodv："+resultSet.getString("accasoa_name"));
                log.info("creditamount："+resultSet.getBigDecimal("creditamount"));


                ydClient.operateData(YDParam.builder()
                        .appType("APP_ERBDTFS82HOVBPL3NFH0")
                        .systemToken("RRB66F91T97H1WN89QZYC47PKLZO2ZQOUMOQLP")
                        .formUuid("FORM-5654B7C22E7546E9BBE7B4EF46DE08BAXZXU")
                        .formDataJson(JSON.toJSONString(hashMap))
                        .build(), YDConf.FORM_OPERATION.create);
            }
        } catch (Exception e) {
            log.info("数据库连接异常"+e.toString());
            e.printStackTrace();
        } finally {
            // 关闭资源
            try {
                if (resultSet != null) resultSet.close();
                if (statement != null) statement.close();
                if (connection != null) connection.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void getSupplierPay() {
        // 获取当前年
        int year = lastMonth.getYear();
        String month = String.format("%02d",lastMonth.getMonthValue()) ;
        deleteMonthDate("FORM-09CDC75CDC6048379567EC87ED6CE7B6T8PP","textField_m0g4itw4", String.valueOf(year),null);

        LocalDate firstDayOfYear = LocalDate.of(year, 1, 1);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String formattedBeginDate = firstDayOfYear.format(formatter);
        Connection connection = null;
        Statement statement = null;
        ResultSet resultSet = null;

        try {
            // 加载 JDBC 驱动
            Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
            // 建立连接
            connection = DriverManager.getConnection(sqlBean.getUrl(), sqlBean.getUsername(), sqlBean.getPassword());

            // 创建 Statement 对象
            statement = connection.createStatement();

            // 测试sql 执行查询
            String sql = "EXECUTE GetAccountDetails  @IncrementFlag = 0, @BeginDate ='"+formattedBeginDate+"'  ";


            resultSet = statement.executeQuery(sql);

            int count = 1;
            while (resultSet.next()) {
                log.info("第{}次",count);
                Map hashMap = new HashMap();

                String years =resultSet.getString("yearv");
                hashMap.put("dateField_lzuyn2l8", UtilDateTime.parse(years,"yyyy"));
                hashMap.put("textField_m0g4itw4", years);
                hashMap.put("textField_m0g4itw6",resultSet.getString("SupplierCode"));
                hashMap.put("textField_m0g4itw5",resultSet.getString("SupplierName"));
                hashMap.put("numberField_m0g4itw7",resultSet.getBigDecimal("BeginMoney")); //期初金额
                hashMap.put("numberField_m0g4itw8",resultSet.getBigDecimal("InvoiceMoney")); //本年到票金额
                hashMap.put("numberField_m0g4itw9",resultSet.getBigDecimal("PayMoney"));//到票已付累计
                hashMap.put("numberField_m0g4itwb",resultSet.getBigDecimal("SumPayMoney")); //到票未付累计
                hashMap.put("numberField_m0g4itwa",resultSet.getBigDecimal("EstimationMoney")); //暂估金额

                ydClient.operateData(YDParam.builder()
                        .appType("APP_ERBDTFS82HOVBPL3NFH0")
                        .systemToken("RRB66F91T97H1WN89QZYC47PKLZO2ZQOUMOQLP")
                        .formUuid("FORM-09CDC75CDC6048379567EC87ED6CE7B6T8PP")
                        .formDataJson(JSON.toJSONString(hashMap))
                        .build(), YDConf.FORM_OPERATION.create);
                count++;
            }
        } catch (Exception e) {
            log.info("数据库连接异常"+e.toString());
            e.printStackTrace();
        } finally {
            // 关闭资源
            try {
                resultSet.close();
                statement.close();
                connection.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
    private void deleteMonthDate(String formUuid,String key, String year, String month){
        String yearAndMonth = year;
        if (ObjectUtil.isNotNull(month)){
            yearAndMonth = yearAndMonth + "-" + month;
        }
        List<Map> allProjectList = ydService.queryFormData_all(YDParam.builder()
                .appType("APP_ERBDTFS82HOVBPL3NFH0")
                .systemToken("RRB66F91T97H1WN89QZYC47PKLZO2ZQOUMOQLP")
                .formUuid(formUuid)
                .searchFieldJson(JSON.toJSONString(UtilMap.map(key, yearAndMonth)))
                .build());
        allProjectList.forEach(e->{
            ydClient.operateData(YDParam.builder()
                    .appType("APP_ERBDTFS82HOVBPL3NFH0")
                    .systemToken("RRB66F91T97H1WN89QZYC47PKLZO2ZQOUMOQLP")
                    .formInstanceId(e.get("formInstanceId").toString())
                    .build(), YDConf.FORM_OPERATION.delete);
        });

    }
}
